#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: skgmm.py
# Date: Wed Oct 29 18:38:36 2014 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import operator
import numpy as np
from sklearn.mixture import GaussianMixture as GMM


class GMMSet(object):
    def __init__(self, gmm_order=32):
        self.gmms = []
        self.gmm_order = gmm_order
        self.y = []

    def fit_new(self, x, label):
        self.y.append(label)
        gmm = GMM(self.gmm_order)
        gmm.fit(x)
        self.gmms.append(gmm)

    def gmm_score(self, gmm, x):
        return np.sum(gmm.score(x))

    def before_pickle(self):
        pass

    def after_pickle(self):
        pass

    def predict_label(self, x):
        scores = self.predict_scores(x)
        p = sorted(enumerate(scores), key=operator.itemgetter(1),
                   reverse=True)
        p = [(str(self.y[i]), y, p[0][1] - y) for i, y in p]
        result = [(self.y[index], value) for (index, value) in
                  enumerate(scores)]
        p = max(result, key=operator.itemgetter(1))
        return p[0]

    def predict_all(self, x):
        scores = [self.gmm_score(gmm, x) / len(x) for gmm in self.gmms]
        p = sorted(enumerate(scores), key=operator.itemgetter(1),
                   reverse=True)
        p = [(str(self.y[i]), y, p[0][1] - y) for i, y in p]
        result = [(self.y[index], value) for (index, value) in
                  enumerate(scores)]
        p = max(result, key=operator.itemgetter(1))
        results = {}
        for label, score in result:
            results[label] = score
        return results

    def predict(self, x):
        scores = [self.gmm_score(gmm, x) / len(x) for gmm in self.gmms]
        p = sorted(enumerate(scores), key=operator.itemgetter(1),
                   reverse=True)
        p = [(str(self.y[i]), y, p[0][1] - y) for i, y in p]
        result = [(self.y[index], value) for (index, value) in
                  enumerate(scores)]
        p = max(result, key=operator.itemgetter(1))
        return p

    def predict_scores(self, x):
        scores = [self.gmm_score(gmm, x) / len(x) for gmm in self.gmms]
        return scores
